/// @file       postgresdemo.c
/// @copyright  Copyright (c) 2012 Toradex AG
///             [Software License Agreement]
/// $Date:		2014-04-14 10:00:00 +0530 (Mon, 14 April 2014) $
/// @brief      Example code for using remote Postgres database using libpq C library
/// @target     Colibri Modules [PXAxxx, Tegra Txx, VFxx], WinCE 6.0/7.0 

/// Header files.
#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <libpq-fe.h>

/// Macros.
#define QUERY_STRING_LENGTH		256		
#define CONNECT					1
#define SELECT_QUERY			2
#define INSERT_QUERY			3
#define UPDATE_QUERY			4
#define DELETE_QUERY			5
#define ALTER_QUERY				6
#define EXIT					7

/// Connection string for the remote Postgres database using default Postgres port. 
/// IP address of the server to connect to should be mention here.
#define CONNECTION_STRING	"hostaddr = '10.18.0.184' port='5432' dbname = 'postgres' \
							user = 'postgres' password = 'postgres'";

/// Close the connection and exit.
void CloseConnection(PGconn *postgresConnection)
{
    PQfinish(postgresConnection);
}

int wmain(void)
{
    const char	*conninfo;
    PGconn		*postgresConnection = NULL;
    PGresult	*resultOfQuery;
	DWORD		selectionInput;
    INT			numberOfFields;
    INT			i, j;
	CHAR		sqlInput[QUERY_STRING_LENGTH] = {0};

	while (TRUE)
	{
		printf("\nSelect Operation\n");
		printf("(1)Connect (2)Query row data (3)Insert Query\n");
		printf("(4)Update Query (5)Delete Query (6)Alter Query\n");
		printf("(7)Exit & Close connection\n");
		printf("Please enter your selection (1 - 7):\t");

		scanf_s("%d", &selectionInput);	

		/// Refer link www.postgresql.org/docs/9.1/static/libpq.html. 
		/// for libpq C library API documentation.

		switch (selectionInput)
		{
			case CONNECT:	
				{
					conninfo = CONNECTION_STRING;
					/// Make a connection to the database. 
					postgresConnection = PQconnectdb(conninfo);
					/// Check to see that the backend connection was successfully made. 
					if (PQstatus(postgresConnection) != CONNECTION_OK)
					{
						fprintf(stderr, "Connection to database failed: %s", PQerrorMessage(postgresConnection));
						printf("\nPress enter to exit.\n");
						fflush(stdin);
						getchar();
						return FALSE;
					} 
					else
					{
						printf("\nConnection successful\n");
					}
					break;
				}
			case SELECT_QUERY:
				{
					/// Example SELECT Query: SELECT * FROM ceschema.cetable
					sqlInput[0] = 0;	
					strcat_s(sqlInput, QUERY_STRING_LENGTH, "DECLARE myportal CURSOR FOR ");
									
					printf("\nEnter the SELECT query: ");
					scanf_s(" %[^\n]", &sqlInput[strlen(sqlInput)]);

					/// Start a transaction block. 
					resultOfQuery = PQexec(postgresConnection, "BEGIN");
					if (PQresultStatus(resultOfQuery) != PGRES_COMMAND_OK)
					{
						fprintf(stderr, "BEGIN command failed: %s", PQerrorMessage(postgresConnection));
						PQclear(resultOfQuery);		
						break;
					}
					
					/// Should PQclear PGresult whenever it is no longer needed to avoid memory leaks.	
					PQclear(resultOfQuery);
					
					/// Fetch rows from the database.	
					resultOfQuery = PQexec(postgresConnection, sqlInput);
					if (PQresultStatus(resultOfQuery) != PGRES_COMMAND_OK)
					{
						fprintf(stderr, "DECLARE CURSOR failed: %s", PQerrorMessage(postgresConnection));
						PQclear(resultOfQuery);

						resultOfQuery = PQexec(postgresConnection, "END");
						PQclear(resultOfQuery);
						break;
					}
					PQclear(resultOfQuery);

					resultOfQuery = PQexec(postgresConnection, "FETCH ALL in myportal");
					if (PQresultStatus(resultOfQuery) != PGRES_TUPLES_OK)
					{
						fprintf(stderr, "FETCH ALL failed: %s", PQerrorMessage(postgresConnection));
						PQclear(resultOfQuery);

						/// Close the portal.
						resultOfQuery = PQexec(postgresConnection, "CLOSE myportal");
						PQclear(resultOfQuery);

						/// End the transaction.
						resultOfQuery = PQexec(postgresConnection, "END");
						PQclear(resultOfQuery);
						break;
					}   
				    
					numberOfFields = PQnfields(resultOfQuery);
					for (i = 0; i < numberOfFields; i++)
					{
						printf("%-15s", PQfname(resultOfQuery, i));
					}
					printf("\n\n");
				    
					for (i = 0; i < PQntuples(resultOfQuery); i++)
					{
						for (j = 0; j < numberOfFields; j++)
						{
							printf("%-15s", PQgetvalue(resultOfQuery, i, j));
						}
						printf("\n");
					}
					PQclear(resultOfQuery);  

					/// Close the portal.
					resultOfQuery = PQexec(postgresConnection, "CLOSE myportal");
					PQclear(resultOfQuery);

					/// End the transaction.
					resultOfQuery = PQexec(postgresConnection, "END");
					PQclear(resultOfQuery);
					break;
				}
			case INSERT_QUERY:	
				{
					/// Example INSERT Query: INSERT INTO ceschema.cetable VALUES ('Chris', 21, 8, 1967)
					sqlInput[0] = 0;				
					
					printf("\nEnter the INSERT query: ");
					scanf_s(" %[^\n]", &sqlInput);

					resultOfQuery = PQexec(postgresConnection, sqlInput);
					if (PQresultStatus(resultOfQuery) != PGRES_COMMAND_OK)
					{
						fprintf(stderr, "Insert query failed: %s", PQerrorMessage(postgresConnection));									
					}
					else
					{
						printf("INSERT query successful\n");
					}
					PQclear(resultOfQuery); 				
					break;
				}
			case UPDATE_QUERY:
				{
					/// Example UPDATE Query: UPDATE ceschema.cetable SET "Date" = 4 WHERE "Year" = 1967
					sqlInput[0] = 0;				
					
					printf("\nEnter the UPDATE query: ");
					scanf_s(" %[^\n]", &sqlInput);

					resultOfQuery = PQexec(postgresConnection, sqlInput);
					if (PQresultStatus(resultOfQuery) != PGRES_COMMAND_OK)
					{
						fprintf(stderr, "Update query failed: %s", PQerrorMessage(postgresConnection));					
					}  
					else
					{
						printf("UPDATE query successful\n");
					}
					PQclear(resultOfQuery); 
					break;
				}
			case DELETE_QUERY:
				{
					/// Example DELETE Query: DELETE FROM ceschema.cetable WHERE "Year"=1967 
					sqlInput[0] = 0;				
					
					printf("\nEnter the DELETE query: ");
					scanf_s(" %[^\n]", &sqlInput);

					resultOfQuery = PQexec(postgresConnection, sqlInput);
					if (PQresultStatus(resultOfQuery) != PGRES_COMMAND_OK)
					{
						fprintf(stderr, "Delete query failed: %s", PQerrorMessage(postgresConnection));					
					}  
					else
					{
						printf("DELETE query successful\n");
					}
					PQclear(resultOfQuery); 
					break;
				}
			case ALTER_QUERY:
				{
					/// Example ALTER Query: ALTER TABLE ceschema.cetable ADD COLUMN Address varchar(50)
					/// PostgreSQL converts all table column names into lowercase, unless quoted.
					/// So in the query, use: "Address"(with quotes) for Address and Address(without quotes) for address.
					sqlInput[0] = 0;				
					
					printf("\nEnter the ALTER query: ");
					scanf_s(" %[^\n]", &sqlInput);

					resultOfQuery = PQexec(postgresConnection, sqlInput);
					if (PQresultStatus(resultOfQuery) != PGRES_COMMAND_OK)
					{
						fprintf(stderr, "Alter query failed: %s", PQerrorMessage(postgresConnection));					
					}  
					else
					{
						printf("ALTER query successful\n");
					}
					PQclear(resultOfQuery); 
					break;
				}
			case EXIT:
				{
					CloseConnection(postgresConnection);
					return TRUE;
				}
			default:
				{
					break;
				}
		}
	}
    return TRUE;
}